
USE [AdventureWorks2012]
GO
/* Create a partition function */
IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO

CREATE PARTITION FUNCTION Demo1_FN (int)
AS RANGE FOR VALUES (100,200,300); -- RANGE LEFT

/* 3 range values; 4 paritions 
{min.....100}, {101....200}, {201....300}, {301....max} 
   Partition1 , Partition2  , Partition3 ,  Partition4
*/
 
SELECT $PARTITION.Demo1_FN(null) AS PartitionNumber -- Null goes left most partition
SELECT $PARTITION.Demo1_FN(-1) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(100) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(101) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(2147483647) AS PartitionNumber


IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO
CREATE PARTITION FUNCTION Demo1_FN (int)
AS RANGE RIGHT FOR VALUES (100,200,300); -- RANGE RIGHT
/* 3 range values; 4 partitions 
{min....99}, {100....199}, {200....299}, {300....max} 
Partition1 , Partition2  , Partition3 ,  Partition4
*/

SELECT $PARTITION.Demo1_FN(null) AS PartitionNumber -- Null goes left most partition
SELECT $PARTITION.Demo1_FN(-1) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(100) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(101) AS PartitionNumber
SELECT $PARTITION.Demo1_FN(2147483647) AS PartitionNumber

/* Lets play with dates */
IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO
CREATE PARTITION FUNCTION Demo1_FN (datetime)
AS RANGE RIGHT FOR VALUES ('20120701','20120801','20120901'); -- RANGE RIGHT

SELECT $PARTITION.Demo1_FN('07/01/2012 00:00:00.000') AS PartitionNumber
SELECT $PARTITION.Demo1_FN('07/01/2012 23:59:59.999') AS PartitionNumber

IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO
CREATE PARTITION FUNCTION Demo1_FN (datetime)
AS RANGE LEFT FOR VALUES ('20120701','20120801','20120901'); /* LEFT LEFT */

/* Which partition does values belong in? */
SELECT $PARTITION.Demo1_FN('07/01/2012 00:00:00.000') AS PartitionNumber
SELECT $PARTITION.Demo1_FN('07/01/2012 00:00:00.003') AS PartitionNumber

/* Fractions of seconds with Range Left */
SELECT CONVERT(datetime, '07/01/2012 23:59:59.997')
SELECT CONVERT(datetime, '07/01/2012 23:59:59.998')
SELECT CONVERT(datetime, '07/01/2012 23:59:59.999')

IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO
CREATE PARTITION FUNCTION Demo1_FN (datetime)
AS RANGE LEFT FOR VALUES ('07/01/2012 23:59:59.997','08/01/2012 23:59:59.997','09/01/2012 23:59:59.997'); /* LEFT RIGHT */

SELECT $PARTITION.Demo1_FN('07/01/2012 23:59:59.998') AS PartitionNumber
SELECT $PARTITION.Demo1_FN('07/01/2012 23:59:59.999') AS PartitionNumber