/** Example eliminating partitions in scan opperations **/
USE [AdventureWorks2012]
GO

DBCC DROPCLEANBUFFERS -- Never do this in production

set statistics io on
set statistics time on
SELECT CustomerID 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged_PT]
WHERE OrderDate > '2011-01-01 00:00:00.000'
AND OrderDate < '2011-12-01 00:00:00.000'

SELECT CustomerID 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged]
WHERE OrderDate > '2011-01-01 00:00:00.000'
AND OrderDate < '2011-12-01 00:00:00.000'
set statistics time off
set statistics io off


/** Example -- Skip-Scan: Seek Keys 

	First, we won't filter any partitions to show that this can have a little worse performance

**/
USE [AdventureWorks2012]
GO

--DROP INDEX idx_SalesOrderHeaderEnlarged_PT_CustomerID ON [Sales].[SalesOrderHeaderEnlarged_PT]
CREATE NONCLUSTERED INDEX idx_SalesOrderHeaderEnlarged_PT_CustomerID ON [Sales].[SalesOrderHeaderEnlarged_PT]
(
	[CustomerID]
) INCLUDE (OrderDate)
ON Demo3_Scheme([OrderDate]);
GO 


CREATE NONCLUSTERED INDEX idx_SalesOrderHeaderEnlarged_CustomerID ON [Sales].[SalesOrderHeaderEnlarged]
(
	[CustomerID]
) INCLUDE (OrderDate)
ON [PRIMARY];
GO

DBCC DROPCLEANBUFFERS; -- Never do this in production

/* Seek all partitions to find CustomerID = 11091 */
set statistics io on
set statistics time on
SELECT MAX(OrderDate) 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged_PT]
WHERE CustomerID =11091;
GO

SELECT MAX(OrderDate) 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged]
WHERE CustomerID =11091;
set statistics time off
set statistics io off

/** Now lets add filter on partition column..
	We only seek in Partition 7 due to OrderDate filter
**/
set statistics io on
set statistics time on
SELECT MAX(OrderDate) 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged_PT]
WHERE CustomerID =11091
AND OrderDate > '2011-01-01 00:00:00.000'
AND OrderDate < '2011-12-01 00:00:00.000';
GO

SELECT MAX(OrderDate) 
FROM [AdventureWorks2012].[Sales].[SalesOrderHeaderEnlarged]
WHERE CustomerID =11091
AND OrderDate > '2011-01-01 00:00:00.000'
AND OrderDate < '2011-12-01 00:00:00.000';
set statistics time off
set statistics io off