USE [AdventureWorks2012]
GO
/* Create a partition function */
IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO

CREATE PARTITION FUNCTION Demo1_FN (int)
AS RANGE LEFT FOR VALUES (100,200,300); 

CREATE PARTITION SCHEME Demo1_Scheme
AS PARTITION Demo1_FN
TO ([FG1], [FG2], [FG3], [FG4]);


SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'

/*
PartitionSchemeName	PartitionID	BoundaryValue	FileGroup
Demo1_Scheme		1			100				FG1
Demo1_Scheme		2			200				FG2
Demo1_Scheme		3			300				FG3
Demo1_Scheme		4			NULL			FG4
*/
ALTER PARTITION FUNCTION Demo1_FN()
MERGE RANGE (200)

SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'

/*
PartitionSchemeName	PartitionID	BoundaryValue	FileGroup
Demo1_Scheme			1			100			 FG1
Demo1_Scheme			2			300			 FG3
Demo1_Scheme			3			NULL		 FG4
*/

/* Mark empty FG as next used */
ALTER PARTITION SCHEME Demo1_Scheme NEXT USED [FG2]

ALTER PARTITION FUNCTION Demo1_FN()
SPLIT RANGE (350) 

SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'

/***** LETS LOOK AT RANGE RIGHT *******/
USE [AdventureWorks2012]
GO

IF EXISTS(select * from sys.partition_schemes where name like 'Demo1_Scheme')
	DROP PARTITION SCHEME [Demo1_Scheme]
IF EXISTS(select * from sys.partition_functions where name like 'Demo1_FN')
	DROP PARTITION FUNCTION [Demo1_FN]
GO

CREATE PARTITION FUNCTION Demo1_FN (int)
AS RANGE RIGHT FOR VALUES (100,200,300); 

CREATE PARTITION SCHEME Demo1_Scheme
AS PARTITION Demo1_FN
TO ([FG1], [FG2], [FG3], [FG4]);

/* lets look at initial partition, boundary value and filegroup associations */
SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'

/*
PartitionSchemeName	PartitionID	BoundaryValue	FileGroup
Demo1_Scheme			1			NULL			FG1
Demo1_Scheme			2			100				FG2
Demo1_Scheme			3			200				FG3
Demo1_Scheme			4			300				FG4
*/

ALTER PARTITION FUNCTION Demo1_FN()
MERGE RANGE (100)

SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'

/*
PartitionSchemeName	PartitionID	BoundaryValue	FileGroup
Demo1_Scheme			1			NULL		 FG1
Demo1_Scheme			2			200			 FG3
Demo1_Scheme			3			300			 FG4
*/

ALTER PARTITION SCHEME Demo1_Scheme NEXT USED [FG2]

ALTER PARTITION FUNCTION Demo1_FN()
SPLIT RANGE (350)

SELECT 
	  SPS.name AS PartitionSchemeName
	, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
		ELSE NULL END AS PartitionID
	, SPRV.value AS BoundaryValue
	, SF.name AS FileGroup
FROM sys.partition_schemes AS SPS
JOIN sys.partition_functions AS SPF 
	ON SPS.function_id = SPF.function_id
JOIN sys.destination_data_spaces AS SDD 
	ON SDD.partition_scheme_id = SPS.data_space_id
JOIN sys.filegroups AS SF 
	ON SF.data_space_id = SDD.data_space_id
LEFT JOIN sys.partition_range_values AS SPRV 
	ON SPRV.function_id = SPF.function_id
	AND SDD.destination_id = 
CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
	ELSE SPRV.boundary_id + 1 END 
WHERE SPS.name like 'Demo1_Scheme'