USE [AdventureWorks2012]
GO

/****** Object:  Table [Sales].[SalesOrderHeaderEnlarged_PT]    Script Date: 10/19/2013 9:08:59 AM ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [Sales].[SalesOrderHeaderEnlarged_PT](
	[SalesOrderID] [int] IDENTITY(1,1) NOT FOR REPLICATION NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderDate] [datetime] NOT NULL,
	[DueDate] [datetime] NOT NULL,
	[ShipDate] [datetime] NULL,
	[Status] [tinyint] NOT NULL,
	[OnlineOrderFlag] [dbo].[Flag] NOT NULL,
	[SalesOrderNumber]  AS (isnull(N'SO'+CONVERT([nvarchar](23),[SalesOrderID],(0)),N'*** ERROR ***')),
	[PurchaseOrderNumber] [dbo].[OrderNumber] NULL,
	[AccountNumber] [dbo].[AccountNumber] NULL,
	[CustomerID] [int] NOT NULL,
	[SalesPersonID] [int] NULL,
	[TerritoryID] [int] NULL,
	[BillToAddressID] [int] NOT NULL,
	[ShipToAddressID] [int] NOT NULL,
	[ShipMethodID] [int] NOT NULL,
	[CreditCardID] [int] NULL,
	[CreditCardApprovalCode] [varchar](15) NULL,
	[CurrencyRateID] [int] NULL,
	[SubTotal] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[TotalDue]  AS (isnull(([SubTotal]+[TaxAmt])+[Freight],(0))),
	[Comment] [nvarchar](128) NULL,
	[rowguid] [uniqueidentifier] ROWGUIDCOL  NOT NULL,
	[ModifiedDate] [datetime] NOT NULL,
 CONSTRAINT [PK_SalesOrderHeaderEnlarged_PT_SalesOrderID] PRIMARY KEY CLUSTERED 
(
	[SalesOrderID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
SET ANSI_PADDING ON
GO

/** Insert the Data **/

SET IDENTITY_INSERT Sales.SalesOrderHeaderEnlarged_PT ON
INSERT INTO Sales.SalesOrderHeaderEnlarged_PT (SalesOrderID, RevisionNumber, OrderDate, DueDate, ShipDate, Status, OnlineOrderFlag, PurchaseOrderNumber, AccountNumber, CustomerID, SalesPersonID, TerritoryID, BillToAddressID, ShipToAddressID, ShipMethodID, CreditCardID, CreditCardApprovalCode, CurrencyRateID, SubTotal, TaxAmt, Freight, Comment, rowguid, ModifiedDate)
SELECT SalesOrderID, RevisionNumber, OrderDate, DueDate, ShipDate, Status, OnlineOrderFlag, PurchaseOrderNumber, AccountNumber, CustomerID, SalesPersonID, TerritoryID, BillToAddressID, ShipToAddressID, ShipMethodID, CreditCardID, CreditCardApprovalCode, CurrencyRateID, SubTotal, TaxAmt, Freight, Comment, rowguid, ModifiedDate
FROM [Sales].[SalesOrderHeaderEnlarged]
SET IDENTITY_INSERT Sales.SalesOrderHeaderEnlarged_PT OFF;

/** Build procedure to get Partition Query Data in Sliding Window example **/
USE [AdventureWorks2012]
GO

CREATE PROCEDURE dbo.PartitionQuery 
(
@SchemaName varchar(200) = 'Demo3_Scheme'
)
AS
BEGIN
	WITH PSCTE AS (SELECT 
		  SPS.name AS PartitionSchemeName
		, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
			ELSE NULL END AS PartitionID
		, SPF.name AS PartitionFunctionName
		,SPF.function_id
		, SPRV.value AS BoundaryValue
		, CASE WHEN SDD.destination_id > SPF.fanout THEN 1 
			ELSE 0 END AS NextUsed
		, SF.name AS FileGroup
	FROM sys.partition_schemes AS SPS
	JOIN sys.partition_functions AS SPF 
		ON SPS.function_id = SPF.function_id
	JOIN sys.destination_data_spaces AS SDD 
		ON SDD.partition_scheme_id = SPS.data_space_id
	JOIN sys.filegroups AS SF 
		ON SF.data_space_id = SDD.data_space_id
	LEFT JOIN sys.partition_range_values AS SPRV 
		ON SPRV.function_id = SPF.function_id
		AND SDD.destination_id = 
	CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
		ELSE SPRV.boundary_id + 1 END 
	WHERE SPS.name like @SchemaName)
	SELECT 
		OBJECT_NAME(SI.object_id) AS PartitionedTable
		,CASE WHEN PF.boundary_value_on_right = 0 THEN 'LEFT' ELSE 'RIGHT' END AS RangeType
		, DS.name AS PartitionSchemeName
		, PF.name AS PartitionFunction
		, P.partition_number AS PartitionNumber
		, P.rows AS PartitionRows
		, FG.name AS FileGroupName
		, SPRV.BoundaryValue AS BoundaryValue
	
	FROM sys.partitions AS P
	JOIN sys.indexes AS SI
		ON P.object_id = SI.object_id AND P.index_id = SI.index_id 
	JOIN sys.data_spaces AS DS
		ON DS.data_space_id = SI.data_space_id
	JOIN sys.partition_schemes AS PS
		ON PS.data_space_id = SI.data_space_id
	JOIN sys.partition_functions AS PF
		ON PF.function_id = PS.function_id 
	JOIN sys.destination_data_spaces AS DDS
		ON DDS.partition_scheme_id = SI.data_space_id 
		AND DDS.destination_id = P.partition_number
	JOIN sys.filegroups AS FG
		ON DDS.data_space_id = FG.data_space_id
	JOIN PSCTE AS SPRV 
		ON (SPRV.PartitionFunctionName = PF.name and SPRV.PartitionID = P.partition_number)
	WHERE DS.type = 'PS'
	AND OBJECTPROPERTYEX(SI.object_id, 'BaseType') = 'U'
	AND SI.type IN(0,1)
	ORDER BY PartitionNumber ASC;
END



GO